@@ -2,49 +2,43 @@
  *       | Inspire Internet Relay Chat Daemon |
  *       +------------------------------------+
  *
- *  InspIRCd: (C) 2002-2008 InspIRCd Development Team
- * See: http://www.inspircd.org/wiki/index.php/Credits
+ *  InspIRCd: (C) 2002-2009 InspIRCd Development Team
+ * See: http://wiki.inspircd.org/Credits
  *
  * This program is free but copyrighted software; see
- *            the file COPYING for details.
+ *	    the file COPYING for details.
  *
  * ---------------------------------------------------
  */
 
 #include "inspircd.h"
-
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
-
-#include "inspircd_config.h"
-#include "configreader.h"
-#include "users.h"
-#include "channels.h"
-#include "modules.h"
-#include "socket.h"
-#include "hashcomp.h"
 #include "transport.h"
+#include "m_cap.h"
 
 #ifdef WINDOWS
 #pragma comment(lib, "libgnutls-13.lib")
-#undef MAX_DESCRIPTORS
-#define MAX_DESCRIPTORS 10000
 #endif
 
 /* $ModDesc: Provides SSL support for clients */
 /* $CompileFlags: pkgconfincludes("gnutls","/gnutls/gnutls.h","") */
 /* $LinkerFlags: rpath("pkg-config --libs gnutls") pkgconflibs("gnutls","/libgnutls.so","-lgnutls") */
 /* $ModDep: transport.h */
+/* $CopyInstall: conf/key.pem $(CONPATH) */
+/* $CopyInstall: conf/cert.pem $(CONPATH) */
 
 enum issl_status { ISSL_NONE, ISSL_HANDSHAKING_READ, ISSL_HANDSHAKING_WRITE, ISSL_HANDSHAKEN, ISSL_CLOSING, ISSL_CLOSED };
 
-bool isin(int port, const std::vector<int> &portlist)
+bool isin(const std::string &host, int port, const std::vector<std::string> &portlist)
 {
-	for(unsigned int i = 0; i < portlist.size(); i++)
-		if(portlist[i] == port)
-			return true;
+	if (std::find(portlist.begin(), portlist.end(), "*:" + ConvToStr(port)) != portlist.end())
+		return true;
 
-	return false;
+	if (std::find(portlist.begin(), portlist.end(), ":" + ConvToStr(port)) != portlist.end())
+		return true;
+
+	return std::find(portlist.begin(), portlist.end(), host + ":" + ConvToStr(port)) != portlist.end();
 }
 
 /** Represents an SSL user's extra data
@@ -56,25 +50,52 @@ public:
 	{
 		sess = NULL;
 	}
+
 	gnutls_session_t sess;
 	issl_status status;
 	std::string outbuf;
-	int inbufoffset;
-	char* inbuf;
-	int fd;
+};
+
+class CommandStartTLS : public Command
+{
+	Module* Caller;
+ public:
+	CommandStartTLS (InspIRCd* Instance, Module* mod) : Command(Instance,"STARTTLS", 0, 0, true), Caller(mod)
+	{
+		this->source = "m_ssl_gnutls.so";
+	}
+
+	CmdResult Handle (const std::vector<std::string> &parameters, User *user)
+	{
+		/* changed from == REG_ALL to catch clients sending STARTTLS
+		 * after NICK and USER but before OnUserConnect completes and
+		 * give a proper error message (see bug #645) - dz
+		 */
+		if (user->registered != REG_NONE)
+		{
+			user->WriteNumeric(691, "%s :STARTTLS is not permitted after client registration has started", user->nick.c_str());
+		}
+		else
+		{
+			if (!user->GetIOHook())
+			{
+				user->WriteNumeric(670, "%s :STARTTLS successful, go ahead with TLS handshake", user->nick.c_str());
+				user->AddIOHook(Caller);
+				Caller->OnRawSocketAccept(user->GetFd(), user->GetIPString(), user->GetPort());
+			}
+			else
+				user->WriteNumeric(691, "%s :STARTTLS failure", user->nick.c_str());
+		}
+
+		return CMD_FAILURE;
+	}
 };
 
 class ModuleSSLGnuTLS : public Module
 {
+	std::vector<std::string> listenports;
 
-	ConfigReader* Conf;
-
-	char* dummy;
-
-	std::vector<int> listenports;
-
-	int inbufsize;
-	issl_session sessions[MAX_DESCRIPTORS];
+	issl_session* sessions;
 
 	gnutls_certificate_credentials x509_cred;
 	gnutls_dh_params dh_params;
@@ -89,47 +110,62 @@ class ModuleSSLGnuTLS : public Module
 	int clientactive;
 	bool cred_alloc;
 
+	CommandStartTLS* starttls;
+
  public:
 
 	ModuleSSLGnuTLS(InspIRCd* Me)
 		: Module(Me)
 	{
-		ServerInstance->PublishInterface("InspSocketHook", this);
+		ServerInstance->Modules->PublishInterface("BufferedSocketHook", this);
 
-		// Not rehashable...because I cba to reduce all the sizes of existing buffers.
-		inbufsize = ServerInstance->Config->NetBufferSize;
+		sessions = new issl_session[ServerInstance->SE->GetMaxFds()];
 
 		gnutls_global_init(); // This must be called once in the program
-		
+
 		cred_alloc = false;
 		// Needs the flag as it ignores a plain /rehash
-		OnRehash(NULL,"ssl");
+		OnModuleRehash(NULL,"ssl");
 
 		// Void return, guess we assume success
 		gnutls_certificate_set_dh_params(x509_cred, dh_params);
+		Implementation eventlist[] = { I_On005Numeric, I_OnRawSocketConnect, I_OnRawSocketAccept,
+			I_OnRawSocketClose, I_OnRawSocketRead, I_OnRawSocketWrite, I_OnCleanup,
+			I_OnBufferFlushed, I_OnRequest, I_OnUnloadModule, I_OnRehash, I_OnModuleRehash,
+			I_OnPostConnect, I_OnEvent, I_OnHookUserIO };
+		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
+
+		starttls = new CommandStartTLS(ServerInstance, this);
+		ServerInstance->AddCommand(starttls);
 	}
 
-	virtual void OnRehash(userrec* user, const std::string &param)
+	virtual void OnRehash(User* user)
 	{
-		Conf = new ConfigReader(ServerInstance);
-
-		for(unsigned int i = 0; i < listenports.size(); i++)
-		{
-			ServerInstance->Config->DelIOHook(listenports[i]);
-		}
+		ConfigReader Conf(ServerInstance);
 
 		listenports.clear();
 		clientactive = 0;
 		sslports.clear();
 
-		for(int i = 0; i < Conf->Enumerate("bind"); i++)
+		for(int index = 0; index < Conf.Enumerate("bind"); index++)
 		{
 			// For each <bind> tag
-			std::string x = Conf->ReadValue("bind", "type", i);
-			if(((x.empty()) || (x == "clients")) && (Conf->ReadValue("bind", "ssl", i) == "gnutls"))
+			std::string x = Conf.ReadValue("bind", "type", index);
+			if(((x.empty()) || (x == "clients")) && (Conf.ReadValue("bind", "ssl", index) == "gnutls"))
 			{
 				// Get the port we're meant to be listening on with SSL
-				std::string port = Conf->ReadValue("bind", "port", i);
+				std::string port = Conf.ReadValue("bind", "port", index);
+				std::string addr = Conf.ReadValue("bind", "address", index);
+
+				if (!addr.empty())
+				{
+					// normalize address, important for IPv6
+					int portint = 0;
+					irc::sockets::sockaddrs bin;
+					if (irc::sockets::aptosa(addr.c_str(), portint, &bin))
+						irc::sockets::satoap(&bin, addr, portint);
+				}
+
 				irc::portparser portrange(port, false);
 				long portno = -1;
 				while ((portno = portrange.GetToken()))
@@ -137,43 +173,46 @@ class ModuleSSLGnuTLS : public Module
 					clientactive++;
 					try
 					{
-						if (ServerInstance->Config->AddIOHook(portno, this))
-						{
-							listenports.push_back(portno);
-							for (size_t i = 0; i < ServerInstance->Config->ports.size(); i++)
-								if (ServerInstance->Config->ports[i]->GetPort() == portno)
-									ServerInstance->Config->ports[i]->SetDescription("ssl");
-							ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: Enabling SSL for port %d", portno);
-							sslports.append("*:").append(ConvToStr(portno)).append(";");
-						}
-						else
-						{
-							ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: FAILED to enable SSL on port %d, maybe you have another ssl or similar module loaded?", portno);
-						}
+						listenports.push_back(addr + ":" + ConvToStr(portno));
+
+						for (size_t i = 0; i < ServerInstance->ports.size(); i++)
+							if ((ServerInstance->ports[i]->GetPort() == portno) && (ServerInstance->ports[i]->GetIP() == addr))
+								ServerInstance->ports[i]->SetDescription("ssl");
+						ServerInstance->Logs->Log("m_ssl_gnutls",DEFAULT, "m_ssl_gnutls.so: Enabling SSL for port %ld", portno);
+
+						if (addr != "127.0.0.1")
+							sslports.append((addr.empty() ? "*" : addr)).append(":").append(ConvToStr(portno)).append(";");
 					}
 					catch (ModuleException &e)
 					{
-						ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: FAILED to enable SSL on port %d: %s. Maybe it's already hooked by the same port on a different IP, or you have an other SSL or similar module loaded?", portno, e.GetReason());
+						ServerInstance->Logs->Log("m_ssl_gnutls",DEFAULT, "m_ssl_gnutls.so: FAILED to enable SSL on port %ld: %s. Maybe it's already hooked by the same port on a different IP, or you have an other SSL or similar module loaded?", portno, e.GetReason());
 					}
 				}
 			}
 		}
 
+		if (!sslports.empty())
+			sslports.erase(sslports.end() - 1);
+	}
+
+	virtual void OnModuleRehash(User* user, const std::string &param)
+	{
 		if(param != "ssl")
-		{
-			delete Conf;
 			return;
-		}
+
+		OnRehash(user);
+
+		ConfigReader Conf(ServerInstance);
 
 		std::string confdir(ServerInstance->ConfigFileName);
 		// +1 so we the path ends with a /
 		confdir = confdir.substr(0, confdir.find_last_of('/') + 1);
 
-		cafile	= Conf->ReadValue("gnutls", "cafile", 0);
-		crlfile	= Conf->ReadValue("gnutls", "crlfile", 0);
-		certfile	= Conf->ReadValue("gnutls", "certfile", 0);
-		keyfile	= Conf->ReadValue("gnutls", "keyfile", 0);
-		dh_bits	= Conf->ReadInteger("gnutls", "dhbits", 0, false);
+		cafile = Conf.ReadValue("gnutls", "cafile", 0);
+		crlfile	= Conf.ReadValue("gnutls", "crlfile", 0);
+		certfile = Conf.ReadValue("gnutls", "certfile", 0);
+		keyfile	= Conf.ReadValue("gnutls", "keyfile", 0);
+		dh_bits	= Conf.ReadInteger("gnutls", "dhbits", 0, false);
 
 		// Set all the default values needed.
 		if (cafile.empty())
@@ -192,20 +231,20 @@ class ModuleSSLGnuTLS : public Module
 			dh_bits = 1024;
 
 		// Prepend relative paths with the path to the config directory.
-		if(cafile[0] != '/')
+		if ((cafile[0] != '/') && (!ServerInstance->Config->StartsWithWindowsDriveLetter(cafile)))
 			cafile = confdir + cafile;
 
-		if(crlfile[0] != '/')
+		if ((crlfile[0] != '/') && (!ServerInstance->Config->StartsWithWindowsDriveLetter(crlfile)))
 			crlfile = confdir + crlfile;
 
-		if(certfile[0] != '/')
+		if ((certfile[0] != '/') && (!ServerInstance->Config->StartsWithWindowsDriveLetter(certfile)))
 			certfile = confdir + certfile;
 
-		if(keyfile[0] != '/')
+		if ((keyfile[0] != '/') && (!ServerInstance->Config->StartsWithWindowsDriveLetter(keyfile)))
 			keyfile = confdir + keyfile;
 
 		int ret;
-		
+
 		if (cred_alloc)
 		{
 			// Deallocate the old credentials
@@ -214,18 +253,18 @@ class ModuleSSLGnuTLS : public Module
 		}
 		else
 			cred_alloc = true;
-		
+
 		if((ret = gnutls_certificate_allocate_credentials(&x509_cred)) < 0)
-			ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: Failed to allocate certificate credentials: %s", gnutls_strerror(ret));
-		
+			ServerInstance->Logs->Log("m_ssl_gnutls",DEBUG, "m_ssl_gnutls.so: Failed to allocate certificate credentials: %s", gnutls_strerror(ret));
+
 		if((ret = gnutls_dh_params_init(&dh_params)) < 0)
-			ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: Failed to initialise DH parameters: %s", gnutls_strerror(ret));
-		
+			ServerInstance->Logs->Log("m_ssl_gnutls",DEBUG, "m_ssl_gnutls.so: Failed to initialise DH parameters: %s", gnutls_strerror(ret));
+
 		if((ret =gnutls_certificate_set_x509_trust_file(x509_cred, cafile.c_str(), GNUTLS_X509_FMT_PEM)) < 0)
-			ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: Failed to set X.509 trust file '%s': %s", cafile.c_str(), gnutls_strerror(ret));
+			ServerInstance->Logs->Log("m_ssl_gnutls",DEBUG, "m_ssl_gnutls.so: Failed to set X.509 trust file '%s': %s", cafile.c_str(), gnutls_strerror(ret));
 
 		if((ret = gnutls_certificate_set_x509_crl_file (x509_cred, crlfile.c_str(), GNUTLS_X509_FMT_PEM)) < 0)
-			ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: Failed to set X.509 CRL file '%s': %s", crlfile.c_str(), gnutls_strerror(ret));
+			ServerInstance->Logs->Log("m_ssl_gnutls",DEBUG, "m_ssl_gnutls.so: Failed to set X.509 CRL file '%s': %s", crlfile.c_str(), gnutls_strerror(ret));
 
 		if((ret = gnutls_certificate_set_x509_key_file (x509_cred, certfile.c_str(), keyfile.c_str(), GNUTLS_X509_FMT_PEM)) < 0)
 		{
@@ -235,8 +274,6 @@ class ModuleSSLGnuTLS : public Module
 
 		// This may be on a large (once a day or week) timer eventually.
 		GenerateDHParams();
-
-		DELETE(Conf);
 	}
 
 	void GenerateDHParams()
@@ -249,7 +286,7 @@ class ModuleSSLGnuTLS : public Module
 		int ret;
 
 		if((ret = gnutls_dh_params_generate2(dh_params, dh_bits)) < 0)
-			ServerInstance->Log(DEFAULT, "m_ssl_gnutls.so: Failed to generate DH parameters (%d bits): %s", dh_bits, gnutls_strerror(ret));
+			ServerInstance->Logs->Log("m_ssl_gnutls",DEFAULT, "m_ssl_gnutls.so: Failed to generate DH parameters (%d bits): %s", dh_bits, gnutls_strerror(ret));
 	}
 
 	virtual ~ModuleSSLGnuTLS()
@@ -257,22 +294,24 @@ class ModuleSSLGnuTLS : public Module
 		gnutls_dh_params_deinit(dh_params);
 		gnutls_certificate_free_credentials(x509_cred);
 		gnutls_global_deinit();
-		ServerInstance->UnpublishInterface("InspSocketHook", this);
+		ServerInstance->Modules->UnpublishInterface("BufferedSocketHook", this);
+		delete[] sessions;
 	}
 
 	virtual void OnCleanup(int target_type, void* item)
 	{
 		if(target_type == TYPE_USER)
 		{
-			userrec* user = (userrec*)item;
+			User* user = (User*)item;
 
-			if(user->GetExt("ssl", dummy) && isin(user->GetPort(), listenports))
+			if (user->GetIOHook() == this)
 			{
 				// User is using SSL, they're a local user, and they're using one of *our* SSL ports.
 				// Potentially there could be multiple SSL modules loaded at once on different ports.
-				userrec::QuitUser(ServerInstance, user, "SSL module unloading");
+				ServerInstance->Users->QuitUser(user, "SSL module unloading");
+				user->DelIOHook();
 			}
-			if (user->GetExt("ssl_cert", dummy) && isin(user->GetPort(), listenports))
+			if (user->GetExt("ssl_cert"))
 			{
 				ssl_cert* tofree;
 				user->GetExt("ssl_cert", tofree);
@@ -288,31 +327,36 @@ class ModuleSSLGnuTLS : public Module
 		{
 			for(unsigned int i = 0; i < listenports.size(); i++)
 			{
-				ServerInstance->Config->DelIOHook(listenports[i]);
-				for (size_t j = 0; j < ServerInstance->Config->ports.size(); j++)
-					if (ServerInstance->Config->ports[j]->GetPort() == listenports[i])
-						ServerInstance->Config->ports[j]->SetDescription("plaintext");
+				for (size_t j = 0; j < ServerInstance->ports.size(); j++)
+					if (listenports[i] == (ServerInstance->ports[j]->GetIP()+":"+ConvToStr(ServerInstance->ports[j]->GetPort())))
+						ServerInstance->ports[j]->SetDescription("plaintext");
 			}
 		}
 	}
 
 	virtual Version GetVersion()
 	{
-		return Version(1, 1, 0, 0, VF_VENDOR, API_VERSION);
+		return Version("$Id: m_ssl_gnutls.cpp 11462 2009-08-02 19:29:24Z w00t $", VF_VENDOR, API_VERSION);
 	}
 
-	void Implements(char* List)
-	{
-		List[I_On005Numeric] = List[I_OnRawSocketConnect] = List[I_OnRawSocketAccept] = List[I_OnRawSocketClose] = List[I_OnRawSocketRead] = List[I_OnRawSocketWrite] = List[I_OnCleanup] = 1;
-		List[I_OnBufferFlushed] = List[I_OnRequest] = List[I_OnSyncUserMetaData] = List[I_OnDecodeMetaData] = List[I_OnUnloadModule] = List[I_OnRehash] = List[I_OnWhois] = List[I_OnPostConnect] = 1;
-	}
 
 	virtual void On005Numeric(std::string &output)
 	{
-		output.append(" SSL=" + sslports);
+		if (!sslports.empty())
+			output.append(" SSL=" + sslports);
+		output.append(" STARTTLS");
 	}
 
-	virtual char* OnRequest(Request* request)
+	virtual void OnHookUserIO(User* user, const std::string &targetip)
+	{
+		if (!user->GetIOHook() && isin(targetip,user->GetPort(),listenports))
+		{
+			/* Hook the user with our module */
+			user->AddIOHook(this);
+		}
+	}
+
+	virtual const char* OnRequest(Request* request)
 	{
 		ISHRequest* ISR = (ISHRequest*)request;
 		if (strcmp("IS_NAME", request->GetId()) == 0)
@@ -321,10 +365,10 @@ class ModuleSSLGnuTLS : public Module
 		}
 		else if (strcmp("IS_HOOK", request->GetId()) == 0)
 		{
-			char* ret = "OK";
+			const char* ret = "OK";
 			try
 			{
-				ret = ServerInstance->Config->AddIOHook((Module*)this, (InspSocket*)ISR->Sock) ? (char*)"OK" : NULL;
+				ret = ISR->Sock->AddIOHook((Module*)this) ? "OK" : NULL;
 			}
 			catch (ModuleException &e)
 			{
@@ -334,15 +378,15 @@ class ModuleSSLGnuTLS : public Module
 		}
 		else if (strcmp("IS_UNHOOK", request->GetId()) == 0)
 		{
-			return ServerInstance->Config->DelIOHook((InspSocket*)ISR->Sock) ? (char*)"OK" : NULL;
+			return ISR->Sock->DelIOHook() ? "OK" : NULL;
 		}
 		else if (strcmp("IS_HSDONE", request->GetId()) == 0)
 		{
 			if (ISR->Sock->GetFd() < 0)
-				return (char*)"OK";
+				return "OK";
 
 			issl_session* session = &sessions[ISR->Sock->GetFd()];
-			return (session->status == ISSL_HANDSHAKING_READ || session->status == ISSL_HANDSHAKING_WRITE) ? NULL : (char*)"OK";
+			return (session->status == ISSL_HANDSHAKING_READ || session->status == ISSL_HANDSHAKING_WRITE) ? NULL : "OK";
 		}
 		else if (strcmp("IS_ATTACH", request->GetId()) == 0)
 		{
@@ -351,14 +395,28 @@ class ModuleSSLGnuTLS : public Module
 				issl_session* session = &sessions[ISR->Sock->GetFd()];
 				if (session->sess)
 				{
-					if ((Extensible*)ServerInstance->FindDescriptor(ISR->Sock->GetFd()) == (Extensible*)(ISR->Sock))
+					if ((Extensible*)ServerInstance->SE->GetRef(ISR->Sock->GetFd()) == (Extensible*)(ISR->Sock))
 					{
-						VerifyCertificate(session, (InspSocket*)ISR->Sock);
+						VerifyCertificate(session, (BufferedSocket*)ISR->Sock);
 						return "OK";
 					}
 				}
 			}
 		}
+		else if (strcmp("GET_FP", request->GetId()) == 0)
+		{
+			if (ISR->Sock->GetFd() > -1)
+			{
+				issl_session* session = &sessions[ISR->Sock->GetFd()];
+				if (session->sess)
+				{
+					Extensible* ext = ISR->Sock;
+					ssl_cert* certinfo;
+					if (ext->GetExt("ssl_cert",certinfo))
+						return certinfo->GetFingerprint().c_str();
+				}
+			}
+		}
 		return NULL;
 	}
 
@@ -366,14 +424,14 @@ class ModuleSSLGnuTLS : public Module
 	virtual void OnRawSocketAccept(int fd, const std::string &ip, int localport)
 	{
 		/* Are there any possibilities of an out of range fd? Hope not, but lets be paranoid */
-		if ((fd < 0) || (fd > MAX_DESCRIPTORS))
+		if ((fd < 0) || (fd > ServerInstance->SE->GetMaxFds() - 1))
 			return;
 
 		issl_session* session = &sessions[fd];
 
-		session->fd = fd;
-		session->inbuf = new char[inbufsize];
-		session->inbufoffset = 0;
+		/* For STARTTLS: Don't try and init a session on a socket that already has a session */
+		if (session->sess)
+			return;
 
 		gnutls_init(&session->sess, GNUTLS_SERVER);
 
@@ -381,33 +439,21 @@ class ModuleSSLGnuTLS : public Module
 		gnutls_credentials_set(session->sess, GNUTLS_CRD_CERTIFICATE, x509_cred);
 		gnutls_dh_set_prime_bits(session->sess, dh_bits);
 
-		/* This is an experimental change to avoid a warning on 64bit systems about casting between integer and pointer of different sizes
-		 * This needs testing, but it's easy enough to rollback if need be
-		 * Old: gnutls_transport_set_ptr(session->sess, (gnutls_transport_ptr_t) fd); // Give gnutls the fd for the socket.
-		 * New: gnutls_transport_set_ptr(session->sess, &fd); // Give gnutls the fd for the socket.
-		 *
-		 * With testing this seems to...not work :/
-		 */
-
 		gnutls_transport_set_ptr(session->sess, (gnutls_transport_ptr_t) fd); // Give gnutls the fd for the socket.
 
 		gnutls_certificate_server_set_request(session->sess, GNUTLS_CERT_REQUEST); // Request client certificate if any.
 
-		Handshake(session);
+		Handshake(session, fd);
 	}
 
 	virtual void OnRawSocketConnect(int fd)
 	{
 		/* Are there any possibilities of an out of range fd? Hope not, but lets be paranoid */
-		if ((fd < 0) || (fd > MAX_DESCRIPTORS))
+		if ((fd < 0) || (fd > ServerInstance->SE->GetMaxFds() - 1))
 			return;
 
 		issl_session* session = &sessions[fd];
 
-		session->fd = fd;
-		session->inbuf = new char[inbufsize];
-		session->inbufoffset = 0;
-
 		gnutls_init(&session->sess, GNUTLS_CLIENT);
 
 		gnutls_set_default_priority(session->sess); // Avoid calling all the priority functions, defaults are adequate.
@@ -415,20 +461,20 @@ class ModuleSSLGnuTLS : public Module
 		gnutls_dh_set_prime_bits(session->sess, dh_bits);
 		gnutls_transport_set_ptr(session->sess, (gnutls_transport_ptr_t) fd); // Give gnutls the fd for the socket.
 
-		Handshake(session);
+		Handshake(session, fd);
 	}
 
 	virtual void OnRawSocketClose(int fd)
 	{
 		/* Are there any possibilities of an out of range fd? Hope not, but lets be paranoid */
-		if ((fd < 0) || (fd > MAX_DESCRIPTORS))
+		if ((fd < 0) || (fd > ServerInstance->SE->GetMaxFds()))
 			return;
 
 		CloseSession(&sessions[fd]);
 
 		EventHandler* user = ServerInstance->SE->GetRef(fd);
 
-		if ((user) && (user->GetExt("ssl_cert", dummy)))
+		if ((user) && (user->GetExt("ssl_cert")))
 		{
 			ssl_cert* tofree;
 			user->GetExt("ssl_cert", tofree);
@@ -440,7 +486,7 @@ class ModuleSSLGnuTLS : public Module
 	virtual int OnRawSocketRead(int fd, char* buffer, unsigned int count, int &readresult)
 	{
 		/* Are there any possibilities of an out of range fd? Hope not, but lets be paranoid */
-		if ((fd < 0) || (fd > MAX_DESCRIPTORS))
+		if ((fd < 0) || (fd > ServerInstance->SE->GetMaxFds() - 1))
 			return 0;
 
 		issl_session* session = &sessions[fd];
@@ -456,8 +502,9 @@ class ModuleSSLGnuTLS : public Module
 		{
 			// The handshake isn't finished, try to finish it.
 
-			if(!Handshake(session))
+			if(!Handshake(session, fd))
 			{
+				errno = session->status == ISSL_CLOSING ? EIO : EAGAIN;
 				// Couldn't resume handshake.
 				return -1;
 			}
@@ -465,7 +512,7 @@ class ModuleSSLGnuTLS : public Module
 		else if (session->status == ISSL_HANDSHAKING_WRITE)
 		{
 			errno = EAGAIN;
-			MakePollWrite(session);
+			MakePollWrite(fd);
 			return -1;
 		}
 
@@ -473,57 +520,31 @@ class ModuleSSLGnuTLS : public Module
 
 		if (session->status == ISSL_HANDSHAKEN)
 		{
-			// Is this right? Not sure if the unencrypted data is garaunteed to be the same length.
-			// Read into the inbuffer, offset from the beginning by the amount of data we have that insp hasn't taken yet.
-			int ret = gnutls_record_recv(session->sess, session->inbuf + session->inbufoffset, inbufsize - session->inbufoffset);
+			int ret = gnutls_record_recv(session->sess, buffer, count);
 
-			if (ret == 0)
+			if (ret > 0)
+			{
+				readresult = ret;
+			}
+			else if (ret == 0)
 			{
 				// Client closed connection.
 				readresult = 0;
 				CloseSession(session);
 				return 1;
 			}
-			else if (ret < 0)
+			else if (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED)
 			{
-				if (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED)
-				{
-					errno = EAGAIN;
-					return -1;
-				}
-				else
-				{
-					readresult = 0;
-					CloseSession(session);
-				}
+				errno = EAGAIN;
+				return -1;
 			}
 			else
 			{
-				// Read successfully 'ret' bytes into inbuf + inbufoffset
-				// There are 'ret' + 'inbufoffset' bytes of data in 'inbuf'
-				// 'buffer' is 'count' long
-
-				unsigned int length = ret + session->inbufoffset;
-
-				if(count <= length)
-				{
-					memcpy(buffer, session->inbuf, count);
-					// Move the stuff left in inbuf to the beginning of it
-					memmove(session->inbuf, session->inbuf + count, (length - count));
-					// Now we need to set session->inbufoffset to the amount of data still waiting to be handed to insp.
-					session->inbufoffset = length - count;
-					// Insp uses readresult as the count of how much data there is in buffer, so:
-					readresult = count;
-				}
-				else
-				{
-					// There's not as much in the inbuf as there is space in the buffer, so just copy the whole thing.
-					memcpy(buffer, session->inbuf, length);
-					// Zero the offset, as there's nothing there..
-					session->inbufoffset = 0;
-					// As above
-					readresult = length;
-				}
+				ServerInstance->Logs->Log("m_ssl_gnutls", DEFAULT,
+						"m_ssl_gnutls.so: Error while reading on fd %d: %s",
+						fd, gnutls_strerror(ret));
+				readresult = 0;
+				CloseSession(session);
 			}
 		}
 		else if(session->status == ISSL_CLOSING)
@@ -535,7 +556,7 @@ class ModuleSSLGnuTLS : public Module
 	virtual int OnRawSocketWrite(int fd, const char* buffer, int count)
 	{
 		/* Are there any possibilities of an out of range fd? Hope not, but lets be paranoid */
-		if ((fd < 0) || (fd > MAX_DESCRIPTORS))
+		if ((fd < 0) || (fd > ServerInstance->SE->GetMaxFds() - 1))
 			return 0;
 
 		issl_session* session = &sessions[fd];
@@ -554,8 +575,8 @@ class ModuleSSLGnuTLS : public Module
 		if (session->status == ISSL_HANDSHAKING_WRITE || session->status == ISSL_HANDSHAKING_READ)
 		{
 			// The handshake isn't finished, try to finish it.
-			Handshake(session);
-			errno = EAGAIN;
+			Handshake(session, fd);
+			errno = session->status == ISSL_CLOSING ? EIO : EAGAIN;
 			return -1;
 		}
 
@@ -573,6 +594,9 @@ class ModuleSSLGnuTLS : public Module
 			{
 				if(ret != GNUTLS_E_AGAIN && ret != GNUTLS_E_INTERRUPTED)
 				{
+					ServerInstance->Logs->Log("m_ssl_gnutls", DEFAULT,
+							"m_ssl_gnutls.so: Error while writing to fd %d: %s",
+							fd, gnutls_strerror(ret));
 					CloseSession(session);
 				}
 				else
@@ -586,57 +610,15 @@ class ModuleSSLGnuTLS : public Module
 			}
 		}
 
-		MakePollWrite(session);
+		MakePollWrite(fd);
 
 		/* Who's smart idea was it to return 1 when we havent written anything?
-		 * This fucks the buffer up in InspSocket :p
+		 * This fucks the buffer up in BufferedSocket :p
 		 */
 		return ret < 1 ? 0 : ret;
 	}
 
-	// :kenny.chatspike.net 320 Om Epy|AFK :is a Secure Connection
-	virtual void OnWhois(userrec* source, userrec* dest)
-	{
-		if (!clientactive)
-			return;
-
-		// Bugfix, only send this numeric for *our* SSL users
-		if(dest->GetExt("ssl", dummy) || (IS_LOCAL(dest) &&  isin(dest->GetPort(), listenports)))
-		{
-			ServerInstance->SendWhoisLine(source, dest, 320, "%s %s :is using a secure connection", source->nick, dest->nick);
-		}
-	}
-
-	virtual void OnSyncUserMetaData(userrec* user, Module* proto, void* opaque, const std::string &extname, bool displayable)
-	{
-		// check if the linking module wants to know about OUR metadata
-		if(extname == "ssl")
-		{
-			// check if this user has an swhois field to send
-			if(user->GetExt(extname, dummy))
-			{
-				// call this function in the linking module, let it format the data how it
-				// sees fit, and send it on its way. We dont need or want to know how.
-				proto->ProtoSendMetaData(opaque, TYPE_USER, user, extname, displayable ? "Enabled" : "ON");
-			}
-		}
-	}
-
-	virtual void OnDecodeMetaData(int target_type, void* target, const std::string &extname, const std::string &extdata)
-	{
-		// check if its our metadata key, and its associated with a user
-		if ((target_type == TYPE_USER) && (extname == "ssl"))
-		{
-			userrec* dest = (userrec*)target;
-			// if they dont already have an ssl flag, accept the remote server's
-			if (!dest->GetExt(extname, dummy))
-			{
-				dest->Extend(extname, "ON");
-			}
-		}
-	}
-
-	bool Handshake(issl_session* session)
+	bool Handshake(issl_session* session, int fd)
 	{
 		int ret = gnutls_handshake(session->sess);
 
@@ -655,12 +637,15 @@ class ModuleSSLGnuTLS : public Module
 				{
 					// gnutls_handshake() wants to write() again.
 					session->status = ISSL_HANDSHAKING_WRITE;
-					MakePollWrite(session);
+					MakePollWrite(fd);
 				}
 			}
 			else
 			{
 				// Handshake failed.
+				ServerInstance->Logs->Log("m_ssl_gnutls", DEFAULT,
+						"m_ssl_gnutls.so: Handshake failed on fd %d: %s",
+						fd, gnutls_strerror(ret));
 				CloseSession(session);
 				session->status = ISSL_CLOSING;
 			}
@@ -671,10 +656,10 @@ class ModuleSSLGnuTLS : public Module
 		{
 			// Handshake complete.
 			// This will do for setting the ssl flag...it could be done earlier if it's needed. But this seems neater.
-			userrec* extendme = ServerInstance->FindDescriptor(session->fd);
+			EventHandler *extendme = ServerInstance->SE->GetRef(fd);
 			if (extendme)
 			{
-				if (!extendme->GetExt("ssl", dummy))
+				if (!extendme->GetExt("ssl"))
 					extendme->Extend("ssl", "ON");
 			}
 
@@ -682,50 +667,44 @@ class ModuleSSLGnuTLS : public Module
 			session->status = ISSL_HANDSHAKEN;
 
 			// Finish writing, if any left
-			MakePollWrite(session);
+			MakePollWrite(fd);
 
 			return true;
 		}
 	}
 
-	virtual void OnPostConnect(userrec* user)
+	virtual void OnPostConnect(User* user)
 	{
 		// This occurs AFTER OnUserConnect so we can be sure the
-		// protocol module has propogated the NICK message.
-		if ((user->GetExt("ssl", dummy)) && (IS_LOCAL(user)))
+		// protocol module has propagated the NICK message.
+		if (user->GetIOHook() == this && (IS_LOCAL(user)))
 		{
-			// Tell whatever protocol module we're using that we need to inform other servers of this metadata NOW.
-			std::deque<std::string>* metadata = new std::deque<std::string>;
-			metadata->push_back(user->nick);
-			metadata->push_back("ssl");		// The metadata id
-			metadata->push_back("ON");		// The value to send
-			Event* event = new Event((char*)metadata,(Module*)this,"send_metadata");
-			event->Send(ServerInstance);		// Trigger the event. We don't care what module picks it up.
-			DELETE(event);
-			DELETE(metadata);
-
-			VerifyCertificate(&sessions[user->GetFd()],user);
+			ssl_cert* certdata = VerifyCertificate(&sessions[user->GetFd()],user);
 			if (sessions[user->GetFd()].sess)
 			{
 				std::string cipher = gnutls_kx_get_name(gnutls_kx_get(sessions[user->GetFd()].sess));
 				cipher.append("-").append(gnutls_cipher_get_name(gnutls_cipher_get(sessions[user->GetFd()].sess))).append("-");
 				cipher.append(gnutls_mac_get_name(gnutls_mac_get(sessions[user->GetFd()].sess)));
-				user->WriteServ("NOTICE %s :*** You are connected using SSL cipher \"%s\"", user->nick, cipher.c_str());
+				user->WriteServ("NOTICE %s :*** You are connected using SSL cipher \"%s\"", user->nick.c_str(), cipher.c_str());
 			}
+
+			ServerInstance->PI->SendMetaData(user, TYPE_USER, "ssl", "ON");
+			if (certdata)
+				ServerInstance->PI->SendMetaData(user, TYPE_USER, "ssl_cert", certdata->GetMetaLine().c_str());
 		}
 	}
 
-	void MakePollWrite(issl_session* session)
+	void MakePollWrite(int fd)
 	{
-		//OnRawSocketWrite(session->fd, NULL, 0);
-		EventHandler* eh = ServerInstance->FindDescriptor(session->fd);
+		//OnRawSocketWrite(fd, NULL, 0);
+		EventHandler* eh = ServerInstance->SE->GetRef(fd);
 		if (eh)
 			ServerInstance->SE->WantWrite(eh);
 	}
 
-	virtual void OnBufferFlushed(userrec* user)
+	virtual void OnBufferFlushed(User* user)
 	{
-		if (user->GetExt("ssl"))
+		if (user->GetIOHook() == this)
 		{
 			issl_session* session = &sessions[user->GetFd()];
 			if (session && session->outbuf.size())
@@ -741,21 +720,15 @@ class ModuleSSLGnuTLS : public Module
 			gnutls_deinit(session->sess);
 		}
 
-		if(session->inbuf)
-		{
-			delete[] session->inbuf;
-		}
-
 		session->outbuf.clear();
-		session->inbuf = NULL;
 		session->sess = NULL;
 		session->status = ISSL_NONE;
 	}
 
-	void VerifyCertificate(issl_session* session, Extensible* user)
+	ssl_cert* VerifyCertificate(issl_session* session, Extensible* user)
 	{
 		if (!session->sess || !user)
-			return;
+			return NULL;
 
 		unsigned int status;
 		const gnutls_datum_t* cert_list;
@@ -777,42 +750,14 @@ class ModuleSSLGnuTLS : public Module
 
 		if (ret < 0)
 		{
-			certinfo->data.insert(std::make_pair("error",std::string(gnutls_strerror(ret))));
-			return;
+			certinfo->error = std::string(gnutls_strerror(ret));
+			return certinfo;
 		}
 
-		if (status & GNUTLS_CERT_INVALID)
-		{
-			certinfo->data.insert(std::make_pair("invalid",ConvToStr(1)));
-		}
-		else
-		{
-			certinfo->data.insert(std::make_pair("invalid",ConvToStr(0)));
-		}
-		if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-		{
-			certinfo->data.insert(std::make_pair("unknownsigner",ConvToStr(1)));
-		}
-		else
-		{
-			certinfo->data.insert(std::make_pair("unknownsigner",ConvToStr(0)));
-		}
-		if (status & GNUTLS_CERT_REVOKED)
-		{
-			certinfo->data.insert(std::make_pair("revoked",ConvToStr(1)));
-		}
-		else
-		{
-			certinfo->data.insert(std::make_pair("revoked",ConvToStr(0)));
-		}
-		if (status & GNUTLS_CERT_SIGNER_NOT_CA)
-		{
-			certinfo->data.insert(std::make_pair("trusted",ConvToStr(0)));
-		}
-		else
-		{
-			certinfo->data.insert(std::make_pair("trusted",ConvToStr(1)));
-		}
+		certinfo->invalid = (status & GNUTLS_CERT_INVALID);
+		certinfo->unknownsigner = (status & GNUTLS_CERT_SIGNER_NOT_FOUND);
+		certinfo->revoked = (status & GNUTLS_CERT_REVOKED);
+		certinfo->trusted = !(status & GNUTLS_CERT_SIGNER_NOT_CA);
 
 		/* Up to here the process is the same for X.509 certificates and
 		 * OpenPGP keys. From now on X.509 certificates are assumed. This can
@@ -820,23 +765,23 @@ class ModuleSSLGnuTLS : public Module
 		 */
 		if (gnutls_certificate_type_get(session->sess) != GNUTLS_CRT_X509)
 		{
-			certinfo->data.insert(std::make_pair("error","No X509 keys sent"));
-			return;
+			certinfo->error = "No X509 keys sent";
+			return certinfo;
 		}
 
 		ret = gnutls_x509_crt_init(&cert);
 		if (ret < 0)
 		{
-			certinfo->data.insert(std::make_pair("error",gnutls_strerror(ret)));
-			return;
+			certinfo->error = gnutls_strerror(ret);
+			return certinfo;
 		}
 
 		cert_list_size = 0;
 		cert_list = gnutls_certificate_get_peers(session->sess, &cert_list_size);
 		if (cert_list == NULL)
 		{
-			certinfo->data.insert(std::make_pair("error","No certificate was found"));
-			return;
+			certinfo->error = "No certificate was found";
+			return certinfo;
 		}
 
 		/* This is not a real world example, since we only check the first
@@ -846,40 +791,47 @@ class ModuleSSLGnuTLS : public Module
 		ret = gnutls_x509_crt_import(cert, &cert_list[0], GNUTLS_X509_FMT_DER);
 		if (ret < 0)
 		{
-			certinfo->data.insert(std::make_pair("error",gnutls_strerror(ret)));
-			return;
+			certinfo->error = gnutls_strerror(ret);
+			return certinfo;
 		}
 
 		gnutls_x509_crt_get_dn(cert, name, &name_size);
-
-		certinfo->data.insert(std::make_pair("dn",name));
+		certinfo->dn = name;
 
 		gnutls_x509_crt_get_issuer_dn(cert, name, &name_size);
-
-		certinfo->data.insert(std::make_pair("issuer",name));
+		certinfo->issuer = name;
 
 		if ((ret = gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_MD5, digest, &digest_size)) < 0)
 		{
-			certinfo->data.insert(std::make_pair("error",gnutls_strerror(ret)));
+			certinfo->error = gnutls_strerror(ret);
 		}
 		else
 		{
-			certinfo->data.insert(std::make_pair("fingerprint",irc::hex(digest, digest_size)));
+			certinfo->fingerprint = irc::hex(digest, digest_size);
 		}
 
 		/* Beware here we do not check for errors.
 		 */
-		if ((gnutls_x509_crt_get_expiration_time(cert) < time(0)) || (gnutls_x509_crt_get_activation_time(cert) > time(0)))
+		if ((gnutls_x509_crt_get_expiration_time(cert) < ServerInstance->Time()) || (gnutls_x509_crt_get_activation_time(cert) > ServerInstance->Time()))
 		{
-			certinfo->data.insert(std::make_pair("error","Not activated, or expired certificate"));
+			certinfo->error = "Not activated, or expired certificate";
 		}
 
 		gnutls_x509_crt_deinit(cert);
 
-		return;
+		return certinfo;
 	}
 
+	void OnEvent(Event* ev)
+	{
+		GenericCapHandler(ev, "tls", "tls");
+	}
+
+	void Prioritize()
+	{
+		Module* server = ServerInstance->Modules->Find("m_spanningtree.so");
+		ServerInstance->Modules->SetPriority(this, I_OnPostConnect, PRIORITY_AFTER, &server);
+	}
 };
 
-MODULE_INIT(ModuleSSLGnuTLS);
-
+MODULE_INIT(ModuleSSLGnuTLS)
