@@ -2,8 +2,8 @@
  *       | Inspire Internet Relay Chat Daemon |
  *       +------------------------------------+
  *
- *  InspIRCd: (C) 2002-2008 InspIRCd Development Team
- * See: http://www.inspircd.org/wiki/index.php/Credits
+ *  InspIRCd: (C) 2002-2009 InspIRCd Development Team
+ * See: http://wiki.inspircd.org/Credits
  *
  * This program is free but copyrighted software; see
  *            the file COPYING for details.
@@ -21,7 +21,7 @@ class SpanningTreeUtilities;
 class TreeSocket;
 class Link;
 
-class HandshakeTimer : public InspTimer
+class HandshakeTimer : public Timer
 {
  private:
 	InspIRCd* Instance;
@@ -31,6 +31,7 @@ class HandshakeTimer : public InspTimer
 	int thefd;
  public:
 	HandshakeTimer(InspIRCd* Inst, TreeSocket* s, Link* l, SpanningTreeUtilities* u, int delay);
+	~HandshakeTimer();
 	virtual void Tick(time_t TIME);
 };
 
