@@ -2,8 +2,8 @@
  *       | Inspire Internet Relay Chat Daemon |
  *       +------------------------------------+
  *
- *  InspIRCd: (C) 2002-2008 InspIRCd Development Team
- * See: http://www.inspircd.org/wiki/index.php/Credits
+ *  InspIRCd: (C) 2002-2009 InspIRCd Development Team
+ * See: http://wiki.inspircd.org/Credits
  *
  * This program is free but copyrighted software; see
  *            the file COPYING for details.
@@ -12,14 +12,9 @@
  */
 
 #include "inspircd.h"
-#include "configreader.h"
-#include "users.h"
-#include "channels.h"
-#include "modules.h"
 #include "commands/cmd_whois.h"
 #include "commands/cmd_stats.h"
 #include "socket.h"
-#include "wildcard.h"
 #include "xline.h"
 #include "transport.h"
 #include "socketengine.h"
@@ -31,7 +26,43 @@
 #include "m_spanningtree/treesocket.h"
 #include "m_spanningtree/resolvers.h"
 
-/* $ModDep: m_spanningtree/timesynctimer.h m_spanningtree/resolvers.h m_spanningtree/main.h m_spanningtree/utils.h m_spanningtree/treeserver.h m_spanningtree/link.h m_spanningtree/treesocket.h */
+/* $ModDep: m_spanningtree/resolvers.h m_spanningtree/main.h m_spanningtree/utils.h m_spanningtree/treeserver.h m_spanningtree/link.h m_spanningtree/treesocket.h */
+
+/* Create server sockets off a listener. */
+void ServerSocketListener::OnAcceptReady(const std::string &ipconnectedto, int newsock, const std::string &incomingip)
+{
+	bool found = false;
+	char *ip = (char *)incomingip.c_str(); // XXX ugly cast
+
+	found = (std::find(Utils->ValidIPs.begin(), Utils->ValidIPs.end(), ip) != Utils->ValidIPs.end());
+	if (!found)
+	{
+		for (std::vector<std::string>::iterator i = Utils->ValidIPs.begin(); i != Utils->ValidIPs.end(); i++)
+		{
+			if (*i == "*" || irc::sockets::MatchCIDR(ip, *i))
+			{
+				found = true;
+				break;
+			}
+		}
+
+		if (!found)
+		{
+			this->ServerInstance->SNO->WriteToSnoMask('l', "Server connection from %s denied (no link blocks with that IP address)", ip);
+			ServerInstance->SE->Close(newsock);
+			return;
+		}
+	}
+
+	if (this->GetIOHook())
+	{
+		this->GetIOHook()->OnRawSocketAccept(newsock, incomingip.c_str(), this->bind_port);
+	}
+
+	/* we don't need a pointer to this, creating it stores it in the necessary places */
+	new TreeSocket(this->Utils, this->ServerInstance, newsock, ip, this->GetIOHook());
+	return;
+}
 
 /** Yay for fast searches!
  * This is hundreds of times faster than recursion
@@ -41,6 +72,9 @@
  */
 TreeServer* SpanningTreeUtilities::FindServer(const std::string &ServerName)
 {
+	if (this->ServerInstance->IsSID(ServerName))
+		return this->FindServerID(ServerName);
+
 	server_hash::iterator iter = serverlist.find(ServerName.c_str());
 	if (iter != serverlist.end())
 	{
@@ -52,66 +86,6 @@ TreeServer* SpanningTreeUtilities::FindServer(const std::string &ServerName)
 	}
 }
 
-TreeServer* SpanningTreeUtilities::FindRemoteBurstServer(TreeServer* Server)
-{
-	server_hash::iterator iter = RemoteServersBursting.find(Server->GetName().c_str());
-	if (iter != RemoteServersBursting.end())
-		return iter->second;
-	else
-		return NULL;
-}
-
-TreeSocket* SpanningTreeUtilities::FindBurstingServer(const std::string &ServerName)
-{
-	std::map<irc::string,TreeSocket*>::iterator iter;
-	iter = burstingserverlist.find(ServerName.c_str());
-	if (iter != burstingserverlist.end())
-	{
-		return iter->second;
-	}
-	else
-	{
-		return NULL;
-	}
-}
-
-void SpanningTreeUtilities::SetRemoteBursting(TreeServer* Server, bool bursting)
-{
-	server_hash::iterator iter = RemoteServersBursting.find(Server->GetName().c_str());
-	if (bursting)
-	{
-		if (iter == RemoteServersBursting.end())
-			RemoteServersBursting.insert(make_pair(Server->GetName(), Server));
-		else return;
-	}
-	else
-	{
-		if (iter != RemoteServersBursting.end())
-			RemoteServersBursting.erase(iter);
-		else return;
-	}
-	ServerInstance->Log(DEBUG,"Server %s is %sbursting nicknames", Server->GetName().c_str(), bursting ? "" : "no longer ");
-}
-
-void SpanningTreeUtilities::AddBurstingServer(const std::string &ServerName, TreeSocket* s)
-{
-	std::map<irc::string,TreeSocket*>::iterator iter = burstingserverlist.find(ServerName.c_str());
-	if (iter == burstingserverlist.end())
-		burstingserverlist[ServerName.c_str()] = s;
-}
-
-void SpanningTreeUtilities::DelBurstingServer(TreeSocket* s)
-{
-	 for (std::map<irc::string,TreeSocket*>::iterator iter = burstingserverlist.begin(); iter != burstingserverlist.end(); iter++)
-	 {
-		 if (iter->second == s)
-		 {
-			 burstingserverlist.erase(iter);
-			 return;
-		 }
-	 }
-}
-
 /** Returns the locally connected server we must route a
  * message through to reach server 'ServerName'. This
  * only applies to one-to-one and not one-to-many routing.
@@ -120,7 +94,7 @@ void SpanningTreeUtilities::DelBurstingServer(TreeSocket* s)
  */
 TreeServer* SpanningTreeUtilities::BestRouteTo(const std::string &ServerName)
 {
-	if (ServerName.c_str() == TreeRoot->GetName())
+	if (ServerName.c_str() == TreeRoot->GetName() || ServerName == ServerInstance->Config->GetSID())
 		return NULL;
 	TreeServer* Found = FindServer(ServerName);
 	if (Found)
@@ -129,6 +103,15 @@ TreeServer* SpanningTreeUtilities::BestRouteTo(const std::string &ServerName)
 	}
 	else
 	{
+		// Cheat a bit. This allows for (better) working versions of routing commands with nick based prefixes, without hassle
+		User *u = ServerInstance->FindNick(ServerName);
+		if (u)
+		{
+			Found = FindServer(u->server);
+			if (Found)
+				return Found->GetRoute();
+		}
+
 		return NULL;
 	}
 }
@@ -143,12 +126,21 @@ TreeServer* SpanningTreeUtilities::FindServerMask(const std::string &ServerName)
 {
 	for (server_hash::iterator i = serverlist.begin(); i != serverlist.end(); i++)
 	{
-		if (match(i->first.c_str(),ServerName.c_str()))
+		if (InspIRCd::Match(i->first,ServerName))
 			return i->second;
 	}
 	return NULL;
 }
 
+TreeServer* SpanningTreeUtilities::FindServerID(const std::string &id)
+{
+	server_hash::iterator iter = sidlist.find(id);
+	if (iter != sidlist.end())
+		return iter->second;
+	else
+		return NULL;
+}
+
 /* A convenient wrapper that returns true if a server exists */
 bool SpanningTreeUtilities::IsServer(const std::string &ServerName)
 {
@@ -159,27 +151,10 @@ SpanningTreeUtilities::SpanningTreeUtilities(InspIRCd* Instance, ModuleSpanningT
 {
 	Bindings.clear();
 
-	lines_to_apply = 0;
+	ServerInstance->Logs->Log("m_spanningtree",DEBUG,"***** Using SID for hash: %s *****", ServerInstance->Config->GetSID().c_str());
 
-	this->TreeRoot = new TreeServer(this, ServerInstance, ServerInstance->Config->ServerName, ServerInstance->Config->ServerDesc);
-
-	modulelist* ml = ServerInstance->FindInterface("InspSocketHook");
-
-	/* Did we find any modules? */
-	if (ml)
-	{
-		/* Yes, enumerate them all to find out the hook name */
-		for (modulelist::iterator m = ml->begin(); m != ml->end(); m++)
-		{
-			/* Make a request to it for its name, its implementing
-			 * InspSocketHook so we know its safe to do this
-			 */
-			std::string name = InspSocketNameRequest((Module*)Creator, *m).Send();
-			/* Build a map of them */
-			hooks[name.c_str()] = *m;
-			hooknames.push_back(name);
-		}
-	}
+	this->TreeRoot = new TreeServer(this, ServerInstance, ServerInstance->Config->ServerName, ServerInstance->Config->ServerDesc, ServerInstance->Config->GetSID());
+	this->ServerUser = new FakeUser(ServerInstance, TreeRoot->GetID());
 
 	this->ReadConfiguration(true);
 }
@@ -188,9 +163,9 @@ SpanningTreeUtilities::~SpanningTreeUtilities()
 {
 	for (unsigned int i = 0; i < Bindings.size(); i++)
 	{
-		ServerInstance->SE->DelFd(Bindings[i]);
-		Bindings[i]->Close();
+		delete Bindings[i];
 	}
+
 	while (TreeRoot->ChildCount())
 	{
 		TreeServer* child_server = TreeRoot->GetChild(0);
@@ -201,8 +176,12 @@ SpanningTreeUtilities::~SpanningTreeUtilities()
 			sock->Close();
 		}
 	}
+	
+	// This avoids a collision on reload
+	ServerUser->uuid = TreeRoot->GetID();
 	delete TreeRoot;
-	ServerInstance->InspSocketCull();
+	delete ServerUser;
+	ServerInstance->BufferedSocketCull();
 }
 
 void SpanningTreeUtilities::AddThisServer(TreeServer* server, TreeServerList &list)
@@ -212,27 +191,19 @@ void SpanningTreeUtilities::AddThisServer(TreeServer* server, TreeServerList &li
 }
 
 /* returns a list of DIRECT servernames for a specific channel */
-void SpanningTreeUtilities::GetListOfServersForChannel(chanrec* c, TreeServerList &list, char status, const CUList &exempt_list)
+void SpanningTreeUtilities::GetListOfServersForChannel(Channel* c, TreeServerList &list, char status, const CUList &exempt_list)
 {
-	CUList *ulist;
-	switch (status)
-	{
-		case '@':
-			ulist = c->GetOppedUsers();
-		break;
-		case '%':
-			ulist = c->GetHalfoppedUsers();
-		break;
-		case '+':
-			ulist = c->GetVoicedUsers();
-		break;
-		default:
-			ulist = c->GetUsers();
-		break;
-	}
+	CUList *ulist = c->GetUsers();
+
 	for (CUList::iterator i = ulist->begin(); i != ulist->end(); i++)
 	{
-		if ((i->first->GetFd() < 0) && (exempt_list.find(i->first) == exempt_list.end()))
+		if (IS_LOCAL(i->first))
+			continue;
+
+		if (status && !strchr(c->GetAllPrefixChars(i->first), status))
+			continue;
+
+		if (exempt_list.find(i->first) == exempt_list.end())
 		{
 			TreeServer* best = this->BestRouteTo(i->first->server);
 			if (best)
@@ -251,7 +222,7 @@ bool SpanningTreeUtilities::DoOneToAllButSenderRaw(const std::string &data, cons
 		if (params.size() >= 2)
 		{
 			/* Prefixes */
-			if ((*(params[0].c_str()) == '@') || (*(params[0].c_str()) == '%') || (*(params[0].c_str()) == '+'))
+			if (ServerInstance->Modes->FindPrefix(params[0][0]))
 			{
 				pfx = params[0][0];
 				params[0] = params[0].substr(1, params[0].length()-1);
@@ -259,7 +230,7 @@ bool SpanningTreeUtilities::DoOneToAllButSenderRaw(const std::string &data, cons
 			if ((*(params[0].c_str()) != '#') && (*(params[0].c_str()) != '$'))
 			{
 				// special routing for private messages/notices
-				userrec* d = ServerInstance->FindNick(params[0]);
+				User* d = ServerInstance->FindNick(params[0]);
 				if (d)
 				{
 					std::deque<std::string> par;
@@ -279,13 +250,13 @@ bool SpanningTreeUtilities::DoOneToAllButSenderRaw(const std::string &data, cons
 			}
 			else
 			{
-				chanrec* c = ServerInstance->FindChan(params[0]);
-				userrec* u = ServerInstance->FindNick(prefix);
-				if (c && u)
+				Channel* c = ServerInstance->FindChan(params[0]);
+				User* u = ServerInstance->FindNick(prefix);
+				if (c)
 				{
 					CUList elist;
 					TreeServerList list;
-					FOREACH_MOD(I_OnBuildExemptList, OnBuildExemptList((command == "PRIVMSG" ? MSG_PRIVMSG : MSG_NOTICE), c, u, pfx, elist));
+					FOREACH_MOD(I_OnBuildExemptList, OnBuildExemptList((command == "PRIVMSG" ? MSG_PRIVMSG : MSG_NOTICE), c, u, pfx, elist, params[1]));
 					GetListOfServersForChannel(c,list,pfx,elist);
 
 					for (TreeServerList::iterator i = list.begin(); i != list.end(); i++)
@@ -408,42 +379,55 @@ void SpanningTreeUtilities::RefreshIPCache()
 	ValidIPs.clear();
 	for (std::vector<Link>::iterator L = LinkBlocks.begin(); L != LinkBlocks.end(); L++)
 	{
-		if ((!L->IPAddr.empty()) && (!L->RecvPass.empty()) && (!L->SendPass.empty()) && (!L->Name.empty()) && (L->Port))
+		if (L->IPAddr.empty() || L->RecvPass.empty() || L->SendPass.empty() || L->Name.empty() || !L->Port)
 		{
-			ValidIPs.push_back(L->IPAddr);
-
-			if (L->AllowMask.length())
-				ValidIPs.push_back(L->AllowMask);
-
-			/* Needs resolving */
-			bool ipvalid = true;
-			QueryType start_type = DNS_QUERY_A;
-#ifdef IPV6
-			start_type = DNS_QUERY_AAAA;
-			if (strchr(L->IPAddr.c_str(),':'))
+			if (L->Name.empty())
 			{
-				in6_addr n;
-				if (inet_pton(AF_INET6, L->IPAddr.c_str(), &n) < 1)
-					ipvalid = false;
+				ServerInstance->Logs->Log("m_spanningtree",DEFAULT,"m_spanningtree: Ignoring a malformed link block (all link blocks require a name!)");
 			}
 			else
-#endif
 			{
-				in_addr n;
-				if (inet_aton(L->IPAddr.c_str(),&n) < 1)
-					ipvalid = false;
+				ServerInstance->Logs->Log("m_spanningtree",DEFAULT,"m_spanningtree: Ignoring a link block missing recvpass, sendpass, port or ipaddr.");
 			}
-			if (!ipvalid)
+
+			/* Invalid link block */
+			continue;
+		}
+
+		ValidIPs.push_back(L->IPAddr);
+
+		if (L->AllowMask.length())
+			ValidIPs.push_back(L->AllowMask);
+
+		/* Needs resolving */
+		bool ipvalid = true;
+		QueryType start_type = DNS_QUERY_A;
+#ifdef IPV6
+		start_type = DNS_QUERY_AAAA;
+		if (strchr(L->IPAddr.c_str(),':'))
+		{
+			in6_addr n;
+			if (inet_pton(AF_INET6, L->IPAddr.c_str(), &n) < 1)
+				ipvalid = false;
+		}
+		else
+#endif
+		{
+			in_addr n;
+			if (inet_aton(L->IPAddr.c_str(),&n) < 1)
+				ipvalid = false;
+		}
+
+		if (!ipvalid)
+		{
+			try
+			{
+				bool cached;
+				SecurityIPResolver* sr = new SecurityIPResolver((Module*)this->Creator, this, ServerInstance, L->IPAddr, *L, cached, start_type);
+				ServerInstance->AddResolver(sr, cached);
+			}
+			catch (...)
 			{
-				try
-				{
-					bool cached;
-					SecurityIPResolver* sr = new SecurityIPResolver((Module*)this->Creator, this, ServerInstance, L->IPAddr, *L, cached, start_type);
-					ServerInstance->AddResolver(sr, cached);
-				}
-				catch (...)
-				{
-				}
 			}
 		}
 	}
@@ -452,14 +436,37 @@ void SpanningTreeUtilities::RefreshIPCache()
 void SpanningTreeUtilities::ReadConfiguration(bool rebind)
 {
 	ConfigReader* Conf = new ConfigReader(ServerInstance);
+
+	/* We don't need to worry about these being *unloaded* on the fly, only loaded,
+	 * because we 'use' the interface locking the module in memory.
+	 */
+	hooks.clear();
+	hooknames.clear();
+	modulelist* ml = ServerInstance->Modules->FindInterface("BufferedSocketHook");
+
+	/* Did we find any modules? */
+	if (ml)
+	{
+		/* Yes, enumerate them all to find out the hook name */
+		for (modulelist::iterator m = ml->begin(); m != ml->end(); m++)
+		{
+			/* Make a request to it for its name, its implementing
+			 * BufferedSocketHook so we know its safe to do this
+			 */
+			std::string name = BufferedSocketNameRequest((Module*)Creator, *m).Send();
+			/* Build a map of them */
+			hooks[name.c_str()] = *m;
+			hooknames.push_back(name);
+		}
+	}
+
 	if (rebind)
 	{
 		for (unsigned int i = 0; i < Bindings.size(); i++)
 		{
-			ServerInstance->SE->DelFd(Bindings[i]);
-			Bindings[i]->Close();
+			delete Bindings[i];
 		}
-		ServerInstance->InspSocketCull();
+		ServerInstance->BufferedSocketCull();
 		Bindings.clear();
 
 		for (int j = 0; j < Conf->Enumerate("bind"); j++)
@@ -472,41 +479,40 @@ void SpanningTreeUtilities::ReadConfiguration(bool rebind)
 			{
 				irc::portparser portrange(Port, false);
 				int portno = -1;
+
+				if (IP == "*")
+					IP.clear();
+
 				while ((portno = portrange.GetToken()))
 				{
-					if (IP == "*")
-						IP.clear();
-
 					if ((!transport.empty()) && (hooks.find(transport.c_str()) ==  hooks.end()))
 					{
-						ServerInstance->Log(DEFAULT,"m_spanningtree: WARNING: Can't find transport type '%s' for port %s:%s - maybe you forgot to load it BEFORE m_spanningtree in your config file? - Skipping this port binding", transport.c_str(), IP.c_str(), Port.c_str());
+						throw CoreException("Can't find transport type '"+transport+"' for port "+IP+":"+Port+" - maybe you forgot to load it BEFORE m_spanningtree in your config file?");
 						break;
 					}
 
-					TreeSocket* listener = new TreeSocket(this, ServerInstance, IP.c_str(), portno, true, 10, transport.empty() ? NULL : hooks[transport.c_str()]);
-					if (listener->GetState() == I_LISTENING)
+					ServerSocketListener *listener = new ServerSocketListener(ServerInstance, this, portno, (char *)IP.c_str());
+					if (listener->GetFd() == -1)
 					{
-						ServerInstance->Log(DEFAULT,"m_spanningtree: Binding server port %s:%d successful!", IP.c_str(), portno);
-						Bindings.push_back(listener);
-					}
-					else
-					{
-						ServerInstance->Log(DEFAULT,"m_spanningtree: Warning: Failed to bind server port: %s:%d: %s",IP.c_str(), portno, strerror(errno));
-						listener->Close();
+						delete listener;
+						continue;
 					}
+
+					if (!transport.empty())
+						listener->AddIOHook(hooks[transport.c_str()]);
+
+					Bindings.push_back(listener);
 				}
 			}
 		}
 	}
-	FlatLinks = Conf->ReadFlag("options","flatlinks",0);
-	HideULines = Conf->ReadFlag("options","hideulines",0);
+	FlatLinks = Conf->ReadFlag("security","flatlinks",0);
+	HideULines = Conf->ReadFlag("security","hideulines",0);
 	AnnounceTSChange = Conf->ReadFlag("options","announcets",0);
-	EnableTimeSync = Conf->ReadFlag("timesync","enable",0);
-	MasterTime = Conf->ReadFlag("timesync", "master", 0);
-	ChallengeResponse = !Conf->ReadFlag("options", "disablehmac", 0);
-	quiet_bursts = Conf->ReadFlag("options", "quietbursts", 0);
-	PingWarnTime = Conf->ReadInteger("options", "pingwarning", "0", 0, true);
-	PingFreq = Conf->ReadInteger("options", "serverpingfreq", "0", 0, true);
+	ChallengeResponse = !Conf->ReadFlag("security", "disablehmac", 0);
+	quiet_bursts = Conf->ReadFlag("performance", "quietbursts", 0);
+	PingWarnTime = Conf->ReadInteger("options", "pingwarning", 0, true);
+	PingFreq = Conf->ReadInteger("options", "serverpingfreq", 0, true);
 
 	if (PingFreq == 0)
 		PingFreq = 60;
@@ -527,6 +533,7 @@ void SpanningTreeUtilities::ReadConfiguration(bool rebind)
 		L.Port = Conf->ReadInteger("link", "port", j, true);
 		L.SendPass = Conf->ReadValue("link", "sendpass", j);
 		L.RecvPass = Conf->ReadValue("link", "recvpass", j);
+		L.Fingerprint = Conf->ReadValue("link", "fingerprint", j);
 		L.AutoConnect = Conf->ReadInteger("link", "autoconnect", j, true);
 		L.HiddenFromStats = Conf->ReadFlag("link", "statshidden", j);
 		L.Timeout = Conf->ReadInteger("link", "timeout", j, true);
@@ -536,93 +543,97 @@ void SpanningTreeUtilities::ReadConfiguration(bool rebind)
 
 		if ((!L.Hook.empty()) && (hooks.find(L.Hook.c_str()) ==  hooks.end()))
 		{
-			ServerInstance->Log(DEFAULT,"m_spanningtree: WARNING: Can't find transport type '%s' for link '%s' - maybe you forgot to load it BEFORE m_spanningtree in your config file? Skipping <link> tag completely.",
-			L.Hook.c_str(), L.Name.c_str());
+			throw CoreException("Can't find transport type '"+L.Hook+"' for link '"+assign(L.Name)+"' - maybe you forgot to load it BEFORE m_spanningtree in your config file? Skipping <link> tag completely.");
 			continue;
 
 		}
 
 		// Fix: Only trip autoconnects if this wouldn't delay autoconnect..
-		if (L.NextConnectTime > ((time_t)(time(NULL) + L.AutoConnect)))
-			L.NextConnectTime = time(NULL) + L.AutoConnect;
-		/* Bugfix by brain, do not allow people to enter bad configurations */
-		if (L.Name != ServerInstance->Config->ServerName)
+		if (L.NextConnectTime > ((time_t)(ServerInstance->Time() + L.AutoConnect)))
+			L.NextConnectTime = ServerInstance->Time() + L.AutoConnect;
+
+		if (L.Name.find('.') == std::string::npos)
+			throw CoreException("The link name '"+assign(L.Name)+"' is invalid and must contain at least one '.' character");
+
+		if (L.Name.length() > 64)
+			throw CoreException("The link name '"+assign(L.Name)+"' is longer than 64 characters!");
+
+		if ((!L.IPAddr.empty()) && (!L.RecvPass.empty()) && (!L.SendPass.empty()) && (!L.Name.empty()) && (L.Port))
 		{
-			if ((!L.IPAddr.empty()) && (!L.RecvPass.empty()) && (!L.SendPass.empty()) && (!L.Name.empty()) && (L.Port))
-			{
-				ValidIPs.push_back(L.IPAddr);
+			if (Allow.length())
+				ValidIPs.push_back(Allow);
 
-				if (Allow.length())
-					ValidIPs.push_back(Allow);
+			ValidIPs.push_back(L.IPAddr);
 
-				/* Needs resolving */
-				bool ipvalid = true;
-				QueryType start_type = DNS_QUERY_A;
+			/* Needs resolving */
+			bool ipvalid = true;
+			QueryType start_type = DNS_QUERY_A;
 #ifdef IPV6
-				start_type = DNS_QUERY_AAAA;
-				if (strchr(L.IPAddr.c_str(),':'))
-				{
-					in6_addr n;
-					if (inet_pton(AF_INET6, L.IPAddr.c_str(), &n) < 1)
-						ipvalid = false;
-				}
-				else
-				{
-					in_addr n;
-					if (inet_aton(L.IPAddr.c_str(),&n) < 1)
-						ipvalid = false;
-				}
-#else
-				in_addr n;
-				if (inet_aton(L.IPAddr.c_str(),&n) < 1)
+			start_type = DNS_QUERY_AAAA;
+			if (strchr(L.IPAddr.c_str(),':'))
+			{
+				in6_addr n;
+				if (inet_pton(AF_INET6, L.IPAddr.c_str(), &n) < 1)
 					ipvalid = false;
-#endif
-
-				if (!ipvalid)
-				{
-					try
-					{
-						bool cached;
-						SecurityIPResolver* sr = new SecurityIPResolver((Module*)this->Creator, this, ServerInstance, L.IPAddr, L, cached, start_type);
-						ServerInstance->AddResolver(sr, cached);
-					}
-					catch (...)
-					{
-					}
-				}
-
-				LinkBlocks.push_back(L);
 			}
 			else
 			{
-				if (L.IPAddr.empty())
+				in_addr n;
+				if (inet_aton(L.IPAddr.c_str(),&n) < 1)
+					ipvalid = false;
+			}
+#else
+			in_addr n;
+			if (inet_aton(L.IPAddr.c_str(),&n) < 1)
+				ipvalid = false;
+#endif
+
+			if (!ipvalid)
+			{
+				try
 				{
-					ServerInstance->Log(DEFAULT,"Invalid configuration for server '%s', IP address not defined!",L.Name.c_str());
+					bool cached;
+					SecurityIPResolver* sr = new SecurityIPResolver((Module*)this->Creator, this, ServerInstance, L.IPAddr, L, cached, start_type);
+					ServerInstance->AddResolver(sr, cached);
 				}
-				else if (L.RecvPass.empty())
+				catch (...)
 				{
-					ServerInstance->Log(DEFAULT,"Invalid configuration for server '%s', recvpass not defined!",L.Name.c_str());
-				}
-				else if (L.SendPass.empty())
-				{
-					ServerInstance->Log(DEFAULT,"Invalid configuration for server '%s', sendpass not defined!",L.Name.c_str());
-				}
-				else if (L.Name.empty())
-				{
-					ServerInstance->Log(DEFAULT,"Invalid configuration, link tag without a name!");
-				}
-				else if (!L.Port)
-				{
-					ServerInstance->Log(DEFAULT,"Invalid configuration for server '%s', no port specified!",L.Name.c_str());
 				}
 			}
 		}
 		else
 		{
-			ServerInstance->Log(DEFAULT,"Invalid configuration for server '%s', link tag has the same server name as the local server!",L.Name.c_str());
+			if (L.IPAddr.empty())
+			{
+				L.IPAddr = "*";
+				ValidIPs.push_back("*");
+				ServerInstance->Logs->Log("m_spanningtree",DEFAULT,"Configuration warning: Link block " + assign(L.Name) + " has no IP defined! This will allow any IP to connect as this server, and MAY not be what you want.");
+			}
+
+			if (L.RecvPass.empty())
+			{
+				throw CoreException("Invalid configuration for server '"+assign(L.Name)+"', recvpass not defined!");
+			}
+
+			if (L.SendPass.empty())
+			{
+				throw CoreException("Invalid configuration for server '"+assign(L.Name)+"', sendpass not defined!");
+			}
+
+			if (L.Name.empty())
+			{
+				throw CoreException("Invalid configuration, link tag without a name! IP address: "+L.IPAddr);
+			}
+
+			if (!L.Port)
+			{
+				ServerInstance->Logs->Log("m_spanningtree",DEFAULT,"Configuration warning: Link block " + assign(L.Name) + " has no port defined, you will not be able to /connect it.");
+			}
 		}
+
+		LinkBlocks.push_back(L);
 	}
-	DELETE(Conf);
+	delete Conf;
 }
 
 void SpanningTreeUtilities::DoFailOver(Link* x)
@@ -631,7 +642,7 @@ void SpanningTreeUtilities::DoFailOver(Link* x)
 	{
 		if (x->FailOver == x->Name)
 		{
-			Creator->RemoteMessage(NULL,"FAILOVER: Some muppet configured the failover for server \002%s\002 to point at itself. Not following it!", x->Name.c_str());
+			this->ServerInstance->SNO->WriteToSnoMask('l', "FAILOVER: Some muppet configured the failover for server \002%s\002 to point at itself. Not following it!", x->Name.c_str());
 			return;
 		}
 		Link* TryThisOne = this->FindLink(x->FailOver.c_str());
@@ -640,30 +651,29 @@ void SpanningTreeUtilities::DoFailOver(Link* x)
 			TreeServer* CheckDupe = this->FindServer(x->FailOver.c_str());
 			if (CheckDupe)
 			{
-				ServerInstance->Log(DEBUG,"Skipping existing failover: %s", x->FailOver.c_str());
+				ServerInstance->Logs->Log("m_spanningtree",DEBUG,"Skipping existing failover: %s", x->FailOver.c_str());
 			}
 			else
 			{
-				Creator->RemoteMessage(NULL,"FAILOVER: Trying failover link for \002%s\002: \002%s\002...", x->Name.c_str(), TryThisOne->Name.c_str());
+				this->ServerInstance->SNO->WriteToSnoMask('l', "FAILOVER: Trying failover link for \002%s\002: \002%s\002...", x->Name.c_str(), TryThisOne->Name.c_str());
 				Creator->ConnectServer(TryThisOne);
 			}
 		}
 		else
 		{
-			Creator->RemoteMessage(NULL,"FAILOVER: Invalid failover server specified for server \002%s\002, will not follow!", x->Name.c_str());
+			this->ServerInstance->SNO->WriteToSnoMask('l', "FAILOVER: Invalid failover server specified for server \002%s\002, will not follow!", x->Name.c_str());
 		}
 	}
 }
 
 Link* SpanningTreeUtilities::FindLink(const std::string& name)
 {
-	for (std::vector<Link>::iterator x = LinkBlocks.begin(); x < LinkBlocks.end(); x++)
+	for (std::vector<Link>::iterator x = LinkBlocks.begin(); x != LinkBlocks.end(); x++)
 	{
-		if (ServerInstance->MatchText(x->Name.c_str(), name.c_str()))
+		if (InspIRCd::Match(x->Name.c_str(), name.c_str()))
 		{
 			return &(*x);
 		}
 	}
 	return NULL;
 }
-
